"""Radicale extension views."""

from django.views import generic

from django.contrib.auth import mixins as auth_mixins
from django.contrib.auth.models import Permission


class CalendarDetailView(auth_mixins.LoginRequiredMixin, generic.TemplateView):
    """Calendar detail view."""

    template_name = "modoboa_radicale/calendar_display.html"

    def get_context_data(self, **kwargs):
        """Include extra information."""
        context = super(CalendarDetailView, self).get_context_data(**kwargs)
        permissions = (
            self.request.user.user_permissions.all() |
            Permission.objects.filter(group__user=self.request.user)
        ).select_related("content_type")
        permissions = [
            "{}.{}".format(p.content_type.app_label, p.codename)
            for p in permissions
        ]
        context.update({"selection": "radicale", "permissions": permissions})
        return context



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-31 15:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('modoboa_radicale', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='sharedcalendar',
            name='color',
            field=models.CharField(default='#3a87ad', max_length=7),
        ),
        migrations.AddField(
            model_name='usercalendar',
            name='color',
            field=models.CharField(default='#3a87ad', max_length=7),
        ),
    ]



"""Default Contacts settings."""

import os

PLUGIN_BASE_DIR = os.path.dirname(__file__)

CALENDAR_STATS_FILES = {
    "dev": os.path.join(
        PLUGIN_BASE_DIR, "../frontend/webpack-stats.json"),
    "prod": os.path.join(
        PLUGIN_BASE_DIR, "static/modoboa_radicale/webpack-stats.json")
}


def apply(settings):
    """Modify settings."""
    DEBUG = settings['DEBUG']
    if "webpack_loader" not in settings["INSTALLED_APPS"]:
        settings["INSTALLED_APPS"] += ("webpack_loader", )
    wpl_config = {
        "CALENDAR": {
            "CACHE": not DEBUG,
            "BUNDLE_DIR_NAME": "modoboa_radicale/",
            "STATS_FILE": CALENDAR_STATS_FILES.get("dev" if DEBUG else "prod"),
            "IGNORE": [".+\.hot-update.js", ".+\.map"]
        }
    }
    if "WEBPACK_LOADER" in settings:
        settings["WEBPACK_LOADER"].update(wpl_config)
    else:
        settings["WEBPACK_LOADER"] = wpl_config



"""Calendar backend definition."""

from importlib import import_module

from modoboa.lib.cryptutils import decrypt


class CalendarBackend(object):
    """Base backend class."""

    def __init__(self, calendar=None):
        """Default constructor."""
        self.calendar = calendar

    def create_event(self, event):
        """Create a new event."""
        raise NotImplementedError

    def get_event(self, uid):
        """Retrieve an even using its uid."""
        raise NotImplementedError

    def get_events(self, start, end):
        """Retrieve a list of event."""
        raise NotImplementedError

    def delete_event(self, uid):
        """Delete an event using its uid."""
        raise NotImplementedError


def get_backend(name, *args, **kwargs):
    """Return a backend instance."""
    module = import_module("modoboa_radicale.backends.{}".format(name))
    return getattr(
        module, "{}Backend".format(name.capitalize()))(*args, **kwargs)


def get_backend_from_request(name, request, calendar=None):
    """Return a backend instance from a request."""
    password = decrypt(request.session["password"])
    return get_backend(
        name, request.user.email, password, calendar=calendar)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-03-24 17:34
from __future__ import unicode_literals

from django.db import migrations, models


def set_path(apps, schema_editor):
    """Set path."""
    UserCalendar = apps.get_model("modoboa_radicale", "UserCalendar")
    for calendar in UserCalendar.objects.select_related("mailbox__domain"):
        calendar._path = "{}@{}/{}".format(
            calendar.mailbox.address, calendar.mailbox.domain.name,
            calendar.name
        )
        calendar.save()
    SharedCalendar = apps.get_model("modoboa_radicale", "SharedCalendar")
    for calendar in SharedCalendar.objects.select_related("domain"):
        calendar._path = "{}/{}".format(
            calendar.domain.name, calendar.name)
        calendar.save()


def backwards(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('modoboa_radicale', '0002_auto_20170831_1721'),
    ]

    operations = [
        migrations.AddField(
            model_name='sharedcalendar',
            name='_path',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='usercalendar',
            name='_path',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
        migrations.RunPython(set_path, backwards),
    ]



"""Radicale test mocks."""

from caldav import objects


EV1 = """BEGIN:VCALENDAR
VERSION:2.0
PRODID:-//Example Corp.//CalDAV Client//EN
BEGIN:VEVENT
UID:20010712T182145Z-123401@example.com
DTSTAMP:20060712T182145Z
DTSTART:20060714T170000Z
DTEND:20060715T040000Z
SUMMARY:Bastille Day Party
END:VEVENT
END:VCALENDAR
"""

EV2 = """BEGIN:VCALENDAR
VERSION:2.0
PRODID:-//Example Corp.//CalDAV Client//EN
BEGIN:VEVENT
UID:20010712T182145Z-123401@example.com
DTSTAMP:20070712T182145Z
DTSTART:20070714T170000Z
DTEND:20070715T040000Z
SUMMARY:Bastille Day Party +1year
END:VEVENT
END:VCALENDAR
"""


class Url(object):

    def __init__(self, path):
        self.path = path

    def geturl(self):
        return self.path


class Calendar(object):

    def __init__(self, client, path):
        self.url = Url(path)
        self.client = client

    def add_event(self, data):
        return True

    def event_by_url(self, url):
        return objects.Event(data=EV1, parent=self)

    def date_search(self, start, end):
        return [
            objects.Event(data=EV1, parent=self),
            objects.Event(data=EV2, parent=self)
        ]

    def set_properties(self, properties):
        return True


class DAVClientMock(object):
    """Mock class for DAVClient instance."""

    def calendar(self, path):
        return Calendar(self, path)

    def mkcalendar(self, url):
        return True

    def delete(self, url):
        return True



"""Calendar serializers."""

from django.utils.translation import ugettext as _

from rest_framework import serializers

from modoboa.admin import models as admin_models
from modoboa.lib import fields as lib_fields

from . import backends
from . import models


class CalDAVCalendarMixin(object):
    """Mixin for calendar serializers."""

    def create_remote_calendar(self, calendar):
        """Create caldav calendar."""
        request = self.context["request"]
        backend = backends.get_backend_from_request("caldav_", request)
        backend.create_calendar(calendar.encoded_url)

    def rename_remote_calendar(self, calendar):
        """Rename caldav calendar."""
        request = self.context["request"]
        backend = backends.get_backend_from_request("caldav_", request)
        backend.rename_calendar(calendar)


class UserCalendarSerializer(CalDAVCalendarMixin, serializers.ModelSerializer):
    """User calendar serializer."""

    class Meta:
        model = models.UserCalendar
        fields = ("pk", "name", "color", "path", "full_url")
        read_only_fields = ("pk", "path", "full_url")

    def create(self, validated_data):
        """Use current user."""
        user = self.context["request"].user
        calendar = models.UserCalendar.objects.create(
            mailbox=user.mailbox, **validated_data)
        self.create_remote_calendar(calendar)
        return calendar

    def update(self, instance, validated_data):
        """Update calendar."""
        old_name = instance.name
        for key, value in validated_data.items():
            setattr(instance, key, value)
        instance.save()
        if old_name != instance.name:
            self.rename_remote_calendar(instance)
        return instance


class DomainSerializer(serializers.ModelSerializer):
    """Domain serializer."""

    pk = serializers.IntegerField()
    name = serializers.CharField()

    class Meta:
        model = admin_models.Domain
        fields = ("pk", "name")
        read_only_fields = ("pk", "name", )


class SharedCalendarSerializer(
        CalDAVCalendarMixin, serializers.ModelSerializer):
    """Shared calendar serializer."""

    domain = DomainSerializer()

    class Meta:
        model = models.SharedCalendar
        fields = ("pk", "name", "color", "path", "domain", "full_url")
        read_only_fields = ("pk", "path", "full_url")

    def create(self, validated_data):
        """Create shared calendar."""
        domain = validated_data.pop("domain")
        calendar = models.SharedCalendar(**validated_data)
        calendar.domain_id = domain["pk"]
        calendar.save()
        self.create_remote_calendar(calendar)
        return calendar

    def update(self, instance, validated_data):
        """Update calendar."""
        domain = validated_data.pop("domain")
        old_name = instance.name
        for key, value in validated_data.items():
            setattr(instance, key, value)
        instance.domain_id = domain["pk"]
        instance.save()
        if old_name != instance.name:
            self.rename_remote_calendar(instance)
        return instance


class AttendeeSerializer(serializers.Serializer):
    """Attendee serializer."""

    display_name = serializers.CharField()
    email = serializers.EmailField()


class EventSerializer(serializers.Serializer):
    """Base event serializer (fullcalendar output)."""

    id = serializers.CharField(read_only=True)
    title = serializers.CharField()
    start = serializers.DateTimeField()
    end = serializers.DateTimeField()
    allDay = serializers.BooleanField(default=False)
    color = serializers.CharField(read_only=True)
    description = serializers.CharField(required=False)

    attendees = AttendeeSerializer(many=True, required=False)


class ROEventSerializer(EventSerializer):
    """Event serializer for read operations."""

    def __init__(self, *args, **kwargs):
        """Set calendar field based on type."""
        calendar_type = kwargs.pop("calendar_type")
        super(ROEventSerializer, self).__init__(*args, **kwargs)
        self.fields["calendar"] = (
            UserCalendarSerializer() if calendar_type == "user"
            else SharedCalendarSerializer()
        )


class WritableEventSerializer(EventSerializer):
    """Event serializer for write operations."""

    calendar = serializers.PrimaryKeyRelatedField(
        queryset=models.UserCalendar.objects.none())
    new_calendar_type = serializers.CharField(required=False)

    def __init__(self, *args, **kwargs):
        """Set calendar list."""
        calendar_type = kwargs.pop("calendar_type")
        super(EventSerializer, self).__init__(*args, **kwargs)
        self.update_calendar_field(calendar_type)

    def update_calendar_field(self, calendar_type):
        """Update field based on given type."""
        if calendar_type == "user":
            self.fields["calendar"].queryset = (
                models.UserCalendar.objects.filter(
                    mailbox__user=self.context["request"].user)
            )
        else:
            self.fields["calendar"].queryset = (
                models.SharedCalendar.objects.filter(
                    domain=self.context["request"].user.mailbox.domain)
            )

    def validate(self, data):
        """Make sure dates are present with allDay flag."""
        errors = {}
        if "allDay" in data:
            if "start" not in data:
                errors["start"] = _("This field is required.")
            if "end" not in data:
                errors["end"] = _("This field is required.")
        if errors:
            raise serializers.ValidationError(errors)
        return data


class MailboxSerializer(serializers.ModelSerializer):
    """Mailbox serializer."""

    pk = serializers.IntegerField()
    full_address = lib_fields.DRFEmailFieldUTF8()

    class Meta:
        model = admin_models.Mailbox
        fields = ("pk", "full_address")
        read_only_fields = ("pk", "full_address", )


class AccessRuleSerializer(serializers.ModelSerializer):
    """AccessRule serializer."""

    mailbox = MailboxSerializer()

    class Meta:
        model = models.AccessRule
        fields = ("pk", "mailbox", "calendar", "read", "write")

    def create(self, validated_data):
        """Create access rule."""
        mailbox = validated_data.pop("mailbox")
        rule = models.AccessRule(**validated_data)
        rule.mailbox_id = mailbox["pk"]
        rule.save()
        return rule

    def update(self, instance, validated_data):
        """Update access rule."""
        mailbox = validated_data.pop("mailbox")
        for key, value in validated_data.items():
            setattr(instance, key, value)
        instance.mailbox_id = mailbox["pk"]
        instance.save()
        return instance



"""Radicale urls."""

from rest_framework_nested import routers

from . import viewsets

router = routers.SimpleRouter()
router.register(
    r"user-calendars", viewsets.UserCalendarViewSet,
    base_name="user-calendar")
router.register(
    r"shared-calendars", viewsets.SharedCalendarViewSet,
    base_name="shared-calendar")
router.register(
    r"attendees", viewsets.AttendeeViewSet, base_name="attendee")
router.register(
    r"mailboxes", viewsets.MailboxViewSet, base_name="mailbox")
router.register(
    r"accessrules", viewsets.AccessRuleViewSet, base_name="access-rule")

calendars_router = routers.NestedSimpleRouter(
    router, r"user-calendars", lookup="calendar")
calendars_router.register(
    r"events", viewsets.UserEventViewSet, base_name="event")
shared_calendars_router = routers.NestedSimpleRouter(
    router, r"shared-calendars", lookup="calendar")
shared_calendars_router.register(
    r"events", viewsets.SharedEventViewSet, base_name="event")

urlpatterns = (
    router.urls + calendars_router.urls + shared_calendars_router.urls
)
